; Defines used in this program

; PXE             build a PXE NBP
; COM_LOG         log to COM
; XEN_LOG         log to XEN port


%ifndef PXE
	org 100h
%endif

	; Print a string
	; -> DS:SI = CS:IP
%macro	print	1+
	call 	write_following
	db	%1, 0
%endm

section .text 

start: 

	; save CS:IP into stack (start in AX)
	call	next
next:
	pop	ax
	sub	ax, next - start
	push	ax
	push	cs
	print	"CS:IP %x:%x", 13, 10
	add	sp, 4

	; print SS:SP
	push	ax
	push	sp
	push	ss
	print	"SS:SP %x:%x", 13, 10
	add	sp, 4
	pop	ax

	; make CS=DS=ES
	shr	ax, 4
	mov	cx, cs
	add	ax, cx
%ifndef PXE
	sub	ax, 10h
%endif
	mov	ds, ax
	mov	es, ax
	push	ax
	push	.back
	retf
.back:

	; set stack pointer
	mov	ax, cs
	mov	ss, ax
	mov	sp, 0fffeh

%ifdef COM_LOG
	; initialize com port
	mov	dx, 0
	mov	cx, 0308h
	xor	bx, bx
	mov	ax, 0401h
	int	14h
%endif

	; test pxe presence
	stc
	mov	ax, 5650h
	int	1ah
	jc	pxe_notfound
	cmp	ax, 564eh
	jne	pxe_notfound

	mov	[pxenv_offset], bx
	mov	[pxenv_segment], es

	push	bx
	push	es
	print	"PXE found at %x:%x", 13, 10
	add	sp, 4

	; check PXENV+
	push	es
	pop	ds
	cmp	dword [bx], 'PXEN'
	jne	wrong_signature
	cmp	word [bx+4], 'V+'
	jne	wrong_signature

	print	"PXE signature ok", 13, 10
	push	word [bx+6]
	print	"version %x", 13, 10
	add	sp, 2

	; check checksum
check_cksum:
	xor	cx, cx
	mov	cl, [bx+8]
	xor	dx, dx
	mov	si, bx
.loop:
	cld
	lodsb
	add	dl, al
	loop	.loop
	cmp	dl, 0
	jnz	wrong_cksum

	print "PXE checksum ok", 13, 10

	lds	ax, [bx+0ah]
	mov	[cs:pxecall_offset], ax
	mov	[cs:pxecall_segment], ds

	; try getting request
	push	0	; buf limit
	push	dword 0	; buf
	push	0	; buf size
	push	2	; dhcp ack
	push	1234h	; status

	mov	di, sp
	push	ss
	pop	es
	mov	bx, 71h
	call	far [cs:pxecall]

	push	word [es:di]
	push	ax
	print	"Get info result %x (status %x)", 13, 10
	push	word [es:di+4]
	push	word [es:di+6]
	push	word [es:di+8]
	print	"Buffer now %x:%x len %x", 13, 10
	add	sp, 10

;	mov	cx, [es:di+4]
;	lds	si, [es:di+6]
;	call	dump

	add	sp, 12	; get rid of called structure

%define IP(a,b,c,d) ((d)*1000000h+(c)*10000h+(b)*100h+(a))
%define HTONS(n) (((n)&0ffh)*100h+((n) >> 8))

%ifdef NOT_DEFINED
	; try to get a file size
	push	dword 0
	sub	sp, 124
	push	0
	push	'AB'
	push	dword IP(192,168,0,1)
	push	dword IP(192,168,0,1)
	push	1234h

	mov	di, sp
	push	ss
	pop	es
	mov	bx, 025h	; TFTP get file size
	call	far [cs:pxecall]

	push	word [es:di+138]
	push	word [es:di+140]
	push	word [es:di]
	push	ax
	print	"TFTP size result %x (status %x) len %x%x", 13, 10
	add	sp, 8 + 142
%endif

;	jmp	pxe_notfound

%ifdef RESTART_TEST
restart_loop:
	; try to restart ourself
	push	10 ; reopen delay
	push	0 ; timeout
	push	0 ; server port
	push	0 ; client port
	push	dword IP(0,0,0,0) ; multicast
	push	dword IP(0,0,0,0) ; gateway
	push	dword IP(10,0,2,2) ; destination
	push	0
	push	7c00h
	push	dword 100000
	sub	sp, 128
	mov	di, sp
	call	after_filename
before_filename:
%ifdef RESTART0
	db	"testrestart.1", 0
%else
%ifdef RESTART1
	db	"abortpxe.com", 0
%else
%ifdef RESTART2
	db	"test#456", 0
%else
%error No restart specified
%endif
%endif
%endif
after_filename:
	pop	si
	push	cs
	pop	ds
	push	ss
	pop	es
	mov	cx, (after_filename - before_filename + 1) / 2
	rep	movsw
	push	1234h ; status

	mov	di, sp
	push	ss
	pop	es
	mov	bx, 073h	; TFTP get file size
	call	far [cs:pxecall]

	push	word [es:di]
	push	ax
	print	"RESTART result %x (status %x)", 13, 10
	add	sp, 4 + 30 + 128

	jmp	restart_loop
%endif


pxe_notfound:
wrong_cksum:
wrong_signature:


	; terminate, in PXE loop waiting characters
%ifdef PXE
halt:
%include "poweroff.inc"
	xor	ax, ax
	int	16h
	jmp	halt
%else
	int	20h
%endif

%include "write.inc"

section .data

udp_string	db	"Hello World!"
udp_string_end:

section .bss

pxenv:
pxenv_offset	resw	1
pxenv_segment	resw	1

pxecall:
pxecall_offset	resw	1
pxecall_segment	resw	1

